@@ -1,3 +1,4 @@
+import enum
 import math
 import opcode as _opcode
 import types
@@ -5,6 +6,21 @@ import types
 import bytecode as _bytecode
 
 
+@enum.unique
+class Compare(enum.IntEnum):
+    LT = 0
+    LE = 1
+    EQ = 2
+    NE = 3
+    GT = 4
+    GE = 5
+    IN = 6
+    NOT_IN = 7
+    IS = 8
+    IS_NOT = 9
+    EXC_MATCH = 10
+
+
 UNSET = object()
 
 def const_key(obj):
@@ -68,35 +84,115 @@ def _check_lineno(lineno):
 
 
 class SetLineno:
-    __slots__ = ('lineno',)
+    __slots__ = ('_lineno',)
 
     def __init__(self, lineno):
         _check_lineno(lineno)
-        self.lineno = lineno
+        self._lineno = lineno
+
+    @property
+    def lineno(self):
+        return self._lineno
 
     def __eq__(self, other):
         if not isinstance(other, SetLineno):
             return False
-        return (self.lineno == other.lineno)
+        return (self._lineno == other._lineno)
 
 
 class Label:
     __slots__ = ()
 
 
-class Instr:
-    """Abstract instruction.
+class _Variable:
+    __slots__ = ('name',)
 
-    lineno, name, op and arg attributes can be modified.
-    """
+    def __init__(self, name):
+        self.name = name
 
+    def __eq__(self, other):
+        if type(self) != type(other):
+            return False
+        return (self.name == other.name)
+
+    def __str__(self):
+        return self.name
+
+    def __repr__(self):
+        return '<%s %r>' % (self.__class__.__name__, self.name)
+
+
+class CellVar(_Variable):
+    __slots__ = ()
+
+
+class FreeVar(_Variable):
+    __slots__ = ()
+
+
+def _check_arg_int(name, arg):
+    if not isinstance(arg, int):
+        raise TypeError("operation %s argument must be an int, "
+                        "got %s"
+                        % (name, type(arg).__name__))
+
+    if not(0 <= arg <= 2147483647):
+        raise ValueError("operation %s argument must be in "
+                         "the range 0..2,147,483,647"
+                         % name)
+
+
+class BaseInstr:
     __slots__ = ('_name', '_opcode', '_arg', '_lineno')
 
     def __init__(self, name, arg=UNSET, *, lineno=None):
-        self.set(name, arg, lineno=lineno)
+        self._set(name, arg, lineno)
 
-    def _check(self, name, arg, lineno):
-        # check operation name
+    def _check_arg(self, name, opcode, arg):
+        if opcode >= _opcode.HAVE_ARGUMENT:
+            if arg is UNSET:
+                raise ValueError("operation %s requires an argument" % name)
+        else:
+            if arg is not UNSET:
+                raise ValueError("operation %s has no argument" % name)
+
+        if self._has_jump(opcode):
+            if not isinstance(arg, (Label, _bytecode.BasicBlock)):
+                raise TypeError("operation %s argument type must be "
+                                "Label or BasicBlock, got %s"
+                                % (name, type(arg).__name__))
+
+        elif opcode in _opcode.hasfree:
+            if not isinstance(arg, (CellVar, FreeVar)):
+                raise TypeError("operation %s argument must be CellVar "
+                                "or FreeVar, got %s"
+                                % (name, type(arg).__name__))
+
+        elif (opcode in _opcode.haslocal
+             or opcode in _opcode.hasname):
+            if not isinstance(arg, str):
+                raise TypeError("operation %s argument must be a str, "
+                                "got %s"
+                                % (name, type(arg).__name__))
+
+        elif opcode in _opcode.hasconst:
+            if isinstance(arg, Label):
+                raise ValueError("label argument cannot be used "
+                                 "in %s operation" % name)
+            if isinstance(arg, _bytecode.BasicBlock):
+                raise ValueError("block argument cannot be used "
+                                 "in %s operation" % name)
+
+        elif opcode in _opcode.hascompare:
+            if not isinstance(arg, Compare):
+                raise TypeError("operation %s argument type must be "
+                                "Compare, got %s"
+                                 % (name, type(arg).__name__))
+
+        elif opcode >= _opcode.HAVE_ARGUMENT:
+            _check_arg_int(name, arg)
+
+    def _set(self, name, arg, lineno):
         if not isinstance(name, str):
             raise TypeError("operation name must be a str")
         try:
@@ -108,16 +204,7 @@ class Instr:
         if lineno is not None:
             _check_lineno(lineno)
 
-        if isinstance(arg, Label) and not self._has_jump(opcode):
-            raise ValueError("label argument cannot be used in %s operation"
-                             % name)
-
-    def set(self, name, arg=UNSET, *, lineno=None):
-        """Modify the instruction in-place.
-
-        Replace name, arg and lineno attributes.
-        """
-        self._check(name, arg, lineno)
+        self._check_arg(name, opcode, arg)
 
         opcode = _opcode.opmap[name]
         self._name = name
@@ -125,6 +212,13 @@ class Instr:
         self._arg = arg
         self._lineno = lineno
 
+    def set(self, name, arg=UNSET):
+        """Modify the instruction in-place.
+
+        Replace name and arg attributes. Don't modify lineno.
+        """
+        self._set(name, arg, self._lineno)
+
     def require_arg(self):
         """Does the instruction require an argument?"""
         return (self._opcode >= _opcode.HAVE_ARGUMENT)
@@ -135,14 +229,14 @@ class Instr:
 
     @name.setter
     def name(self, name):
-        self.set(name, self._arg, lineno=self._lineno)
+        self._set(name, self._arg, self._lineno)
 
     @property
-    def op(self):
+    def opcode(self):
         return self._opcode
 
-    @op.setter
-    def op(self, op):
+    @opcode.setter
+    def opcode(self, op):
         if not isinstance(op, int):
             raise TypeError("operator code must be an int")
         if 0 <= op <= 255:
@@ -153,7 +247,7 @@ class Instr:
         if not valid:
             raise ValueError("invalid operator code")
 
-        self.set(name, self._arg, lineno=self._lineno)
+        self._set(name, self._arg, self._lineno)
 
     @property
     def arg(self):
@@ -161,7 +255,7 @@ class Instr:
 
     @arg.setter
     def arg(self, arg):
-        self.set(self._name, arg, lineno=self._lineno)
+        self._set(self._name, arg, self._lineno)
 
     @property
     def lineno(self):
@@ -169,7 +263,7 @@ class Instr:
 
     @lineno.setter
     def lineno(self, lineno):
-        self.set(self._name, self._arg, lineno=lineno)
+        self._set(self._name, self._arg, lineno)
 
     def copy(self):
         return self.__class__(self._name, self._arg, lineno=self._lineno)
@@ -221,3 +315,9 @@ class Instr:
         if self.is_uncond_jump():
             return True
         return False
+
+
+class Instr(BaseInstr):
+    """Abstract instruction."""
+
+    __slots__ = ()
