@@ -2,8 +2,8 @@ import textwrap
 import types
 import unittest
 
-from bytecode import (UNSET, Label, Instr, ConcreteInstr, Block,
-                      Bytecode, BytecodeBlocks, ConcreteBytecode)
+from bytecode import (UNSET, Label, Instr, ConcreteInstr, BasicBlock,
+                      Bytecode, ControlFlowGraph, ConcreteBytecode)
 
 
 def _format_instr_list(block, labels, lineno):
@@ -18,6 +18,8 @@ def _format_instr_list(block, labels, lineno):
             if arg is not UNSET:
                 if isinstance(arg, Label):
                     arg = labels[arg]
+                elif isinstance(arg, BasicBlock):
+                    arg = labels[id(arg)]
                 else:
                     arg = repr(arg)
                 if lineno:
@@ -34,7 +36,7 @@ def _format_instr_list(block, labels, lineno):
         instr_list.append(text)
     return '[%s]'  % ',\n '.join(instr_list)
 
-def dump_code(code, lineno=True):
+def dump_bytecode(code, lineno=True):
     """
     Use this function to write unit tests: copy/paste its output to
     write a self.assertBlocksEqual() check.
@@ -61,8 +63,8 @@ def dump_code(code, lineno=True):
             print(indent + 'code = %s()' % name)
             if code.argcount:
                 print(indent + 'code.argcount = %s' % code.argcount)
-            if code.kw_only_argcount:
-                print(indent + 'code.argcount = %s' % code.kw_only_argcount)
+            if code.kwonlyargcount:
+                print(indent + 'code.argcount = %s' % code.kwonlyargcount)
             print(indent + 'code.flags = %#x' % code.flags)
             if code.consts:
                 print(indent + 'code.consts = %r' % code.consts)
@@ -93,12 +95,13 @@ def dump_code(code, lineno=True):
 
         print()
     else:
+        assert isinstance(code, ControlFlowGraph)
         labels = {}
         for block_index, block in enumerate(code):
-            labels[block.label] = 'code[%s].label' % block_index
+            labels[id(block)] = 'code[%s]' % block_index
 
         for block_index, block in enumerate(code):
-            text = _format_instr_list(block, labels)
+            text = _format_instr_list(block, labels, lineno)
             if block_index != len(code) - 1:
                 text += ','
             print(text)
@@ -115,51 +118,38 @@ def get_code(source, *, filename="<string>", function=False):
         code = sub_code[0]
     return code
 
-def disassemble(source, *, filename="<string>", function=False,
-                remove_last_return_none=False):
+def disassemble(source, *, filename="<string>", function=False):
     code = get_code(source, filename=filename, function=function)
-
-    bytecode = Bytecode.from_code(code)
-    blocks = BytecodeBlocks.from_bytecode(bytecode)
-    if remove_last_return_none:
-        # drop LOAD_CONST+RETURN_VALUE to only keep 2 instructions,
-        # to make unit tests shorter
-        block = blocks[-1]
-        test = (block[-2].name == "LOAD_CONST"
-                and block[-2].arg is None
-                and block[-1].name == "RETURN_VALUE")
-        if not test:
-            raise ValueError("unable to find implicit RETURN_VALUE <None>: %s"
-                             % block[-2:])
-        del block[-2:]
-    return blocks
+    return Bytecode.from_code(code)
 
 
 class TestCase(unittest.TestCase):
     def assertBlocksEqual(self, code, *expected_blocks):
 
-        def _flat(code, blocks):
+        # Code similar to ControlFlowGraph._flat(), but code is a list of
+        # Instr, not a ControlFlowGraph object
+        def _flat(code, expected_blocks):
             instructions = []
             labels = {}
             jumps = []
 
-            offset = 0
-            for block_index, block in enumerate(code, 1):
-                labels[id(block)] = offset
-                for instr in block:
-                    offset += 1
+            for block_index, block in enumerate(code):
+                labels[id(block)] = block_index
 
             for block_index, block in enumerate(expected_blocks, 1):
                 for index, instr in enumerate(block):
-                    if isinstance(instr, Instr) and isinstance(instr.arg, Block):
+                    if (isinstance(instr, Instr)
+                       and isinstance(instr.arg, BasicBlock)):
                         # copy the instruction to be able to modify
                         # its argument below
-                        instr = instr.copy()
-                        jumps.append(instr)
+                        target_block = instr.arg
+                        instr = ConcreteInstr(instr.name, 0,
+                                              lineno=instr.lineno)
+                        jumps.append((target_block, instr))
                     instructions.append(instr)
 
-            for instr in jumps:
-                instr.arg = labels[id(instr.arg)]
+            for block, instr in jumps:
+                instr.arg = labels[id(block)]
 
             return instructions
 
