@@ -1,7 +1,7 @@
 import textwrap
 import types
 import unittest
-from bytecode import Label, Instr, Bytecode, BytecodeBlocks
+from bytecode import Label, Instr, Compare, Bytecode, ControlFlowGraph
 from bytecode import peephole_opt
 from bytecode.tests import TestCase
 from unittest import mock
@@ -12,14 +12,14 @@ class Tests(TestCase):
 
     def optimize_blocks(self, code):
         if isinstance(code, Bytecode):
-            code = BytecodeBlocks.from_bytecode(code)
+            code = ControlFlowGraph.from_bytecode(code)
         optimizer = peephole_opt.PeepholeOptimizer()
         optimizer._optimize(code)
         return code
 
     def check(self, code, *expected):
         if isinstance(code, Bytecode):
-            code = BytecodeBlocks.from_bytecode(code)
+            code = ControlFlowGraph.from_bytecode(code)
         optimizer = peephole_opt.PeepholeOptimizer()
         optimizer._optimize(code)
         code = code.to_bytecode()
@@ -30,7 +30,7 @@ class Tests(TestCase):
             self.assertListEqual(code, list(expected))
 
     def check_dont_optimize(self, code):
-        code = BytecodeBlocks.from_bytecode(code)
+        code = ControlFlowGraph.from_bytecode(code)
         noopt = code.to_bytecode()
 
         optim = self.optimize_blocks(code)
@@ -215,13 +215,13 @@ class Tests(TestCase):
                          Instr('LOAD_CONST', 2),
                          Instr('LOAD_CONST', 3),
                          Instr('BUILD_LIST', 3),
-                         Instr('COMPARE_OP', 6),
+                         Instr('COMPARE_OP', Compare.IN),
                          Instr('STORE_NAME', 'test')])
 
         self.check(code,
                    Instr('LOAD_NAME', 'x'),
                    Instr('LOAD_CONST', (1, 2, 3)),
-                   Instr('COMPARE_OP', 6),
+                   Instr('COMPARE_OP', Compare.IN),
                    Instr('STORE_NAME', 'test'))
 
     def test_build_list_unpack_seq(self):
@@ -268,6 +268,38 @@ class Tests(TestCase):
                        Instr('STORE_NAME', 'y'),
                        Instr('STORE_NAME', 'z'))
 
+    def test_build_tuple_unpack_seq_const(self):
+        # x, y = (3, 4)
+        code = Bytecode([Instr('LOAD_CONST', 3),
+                         Instr('LOAD_CONST', 4),
+                         Instr('BUILD_TUPLE', 2),
+                         Instr('UNPACK_SEQUENCE', 2),
+                         Instr('STORE_NAME', 'x'),
+                         Instr('STORE_NAME', 'y')])
+        self.check(code,
+                   Instr('LOAD_CONST', (3, 4)),
+                   Instr('UNPACK_SEQUENCE', 2),
+                   Instr('STORE_NAME', 'x'),
+                   Instr('STORE_NAME', 'y'))
+
+    def test_build_list_unpack_seq_const(self):
+        # x, y, z = [3, 4, 5]
+        code = Bytecode([Instr('LOAD_CONST', 3),
+                         Instr('LOAD_CONST', 4),
+                         Instr('LOAD_CONST', 5),
+                         Instr('BUILD_LIST', 3),
+                         Instr('UNPACK_SEQUENCE', 3),
+                         Instr('STORE_NAME', 'x'),
+                         Instr('STORE_NAME', 'y'),
+                         Instr('STORE_NAME', 'z')])
+        self.check(code,
+                   Instr('LOAD_CONST', 5),
+                   Instr('LOAD_CONST', 4),
+                   Instr('LOAD_CONST', 3),
+                   Instr('STORE_NAME', 'x'),
+                   Instr('STORE_NAME', 'y'),
+                   Instr('STORE_NAME', 'z'))
+
     def test_build_set(self):
         # test = x in {1, 2, 3}
         code = Bytecode([Instr('LOAD_NAME', 'x'),
@@ -275,22 +307,21 @@ class Tests(TestCase):
                          Instr('LOAD_CONST', 2),
                          Instr('LOAD_CONST', 3),
                          Instr('BUILD_SET', 3),
-                         Instr('COMPARE_OP', 6),
+                         Instr('COMPARE_OP', Compare.IN),
                          Instr('STORE_NAME', 'test')])
 
         self.check(code,
                    Instr('LOAD_NAME', 'x'),
                    Instr('LOAD_CONST', frozenset((1, 2, 3))),
-                   Instr('COMPARE_OP', 6),
+                   Instr('COMPARE_OP', Compare.IN),
                    Instr('STORE_NAME', 'test'))
 
     def test_compare_op_unary_not(self):
-        # FIXME: use constants, not hardcoded values
         for op, not_op in (
-            (6, 7), # in => not in
-            (7, 6), # not in => in
-            (8, 9), # is => is not
-            (9, 8),
+            (Compare.IN, Compare.NOT_IN),   # in => not in
+            (Compare.NOT_IN, Compare.IN),   # not in => in
+            (Compare.IS, Compare.IS_NOT),   # is => is not
+            (Compare.IS_NOT, Compare.IS),   # is not => is
         ):
             code = Bytecode([Instr('LOAD_NAME', 'a'),
                              Instr('LOAD_NAME', 'b'),
@@ -310,7 +341,7 @@ class Tests(TestCase):
                          Instr('JUMP_IF_FALSE_OR_POP', label_instr5),
                          Instr('LOAD_NAME', 'b'),
                          Instr('LOAD_CONST', True),
-                         Instr('COMPARE_OP', 8),
+                         Instr('COMPARE_OP', Compare.IS),
                          label_instr5,
                          Instr('UNARY_NOT'),
                          Instr('STORE_NAME', 'x'),
@@ -322,7 +353,7 @@ class Tests(TestCase):
         # x = 3 < 5
         code = Bytecode([Instr('LOAD_CONST', 3),
                          Instr('LOAD_CONST', 5),
-                         Instr('COMPARE_OP', 0),
+                         Instr('COMPARE_OP', Compare.LT),
                          Instr('STORE_NAME', 'x'),
                          Instr('LOAD_CONST', None),
                          Instr('RETURN_VALUE')])
@@ -369,7 +400,7 @@ class Tests(TestCase):
                          Instr('RETURN_VALUE', lineno=2),
                          Instr('LOAD_CONST', 5, lineno=3),
                          Instr('RETURN_VALUE', lineno=3)])
-        code = BytecodeBlocks.from_bytecode(code)
+        code = ControlFlowGraph.from_bytecode(code)
         self.check(code,
                    Instr('LOAD_CONST', 4, lineno=2),
                    Instr('RETURN_VALUE', lineno=2))
@@ -389,7 +420,7 @@ class Tests(TestCase):
                          Instr('RETURN_VALUE', lineno=4),
                          Instr('LOAD_CONST', 7, lineno=5),
                          Instr('RETURN_VALUE', lineno=5)])
-        code = BytecodeBlocks.from_bytecode(code)
+        code = ControlFlowGraph.from_bytecode(code)
         self.check(code,
                    Instr('LOAD_CONST', 4, lineno=2),
                    Instr('RETURN_VALUE', lineno=2))
@@ -408,7 +439,7 @@ class Tests(TestCase):
                          return_label,
                              Instr('LOAD_CONST', None, lineno=3),
                              Instr('RETURN_VALUE', lineno=3)])
-        code = BytecodeBlocks.from_bytecode(code)
+        code = ControlFlowGraph.from_bytecode(code)
 
         end_loop = Label()
         self.check(code,
@@ -425,27 +456,22 @@ class Tests(TestCase):
         #
         # if not x:
         #     y = 9
-        # y = 4
         label = Label()
         code = Bytecode([Instr('LOAD_NAME', 'x'),
                          Instr('UNARY_NOT'),
                          Instr('POP_JUMP_IF_FALSE', label),
-                         Instr('LOAD_CONST', 9),
-                         Instr('STORE_NAME', 'y'),
-                         label,
-                         Instr('LOAD_CONST', 4),
-                         Instr('STORE_NAME', 'y')])
+                             Instr('LOAD_CONST', 9),
+                             Instr('STORE_NAME', 'y'),
+                         label])
 
         code = self.optimize_blocks(code)
         label = Label()
         self.check(code,
                    Instr('LOAD_NAME', 'x'),
                    Instr('POP_JUMP_IF_TRUE', label),
-                   Instr('LOAD_CONST', 9),
-                   Instr('STORE_NAME', 'y'),
-                   label,
-                   Instr('LOAD_CONST', 4),
-                   Instr('STORE_NAME', 'y'))
+                       Instr('LOAD_CONST', 9),
+                       Instr('STORE_NAME', 'y'),
+                   label)
 
     def test_unconditional_jump_to_return(self):
         source = """
@@ -588,6 +614,28 @@ class Tests(TestCase):
     #    """
     #    XXX
 
+    def test_nop(self):
+        code = Bytecode([Instr('LOAD_NAME', 'x'),
+                         Instr('NOP'),
+                         Instr('STORE_NAME', 'test')])
+
+        self.check(code,
+                   Instr('LOAD_NAME', 'x'),
+                   Instr('STORE_NAME', 'test'))
+
+    def test_dead_code_jump(self):
+        label = Label()
+        code = Bytecode([Instr('LOAD_NAME', 'x'),
+                         Instr('JUMP_ABSOLUTE', label),
+                             # dead code
+                             Instr('NOP'),
+                         label,
+                             Instr('STORE_NAME', 'test')])
+
+        self.check(code,
+                   Instr('LOAD_NAME', 'x'),
+                   Instr('STORE_NAME', 'test'))
+
 
 if __name__ == "__main__":
     unittest.main()
